/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.management;

import java.util.List;

/**
 * Interface for managing the JOnAS services.
 * @author Francois Fornaciari
 */
public interface ServiceManager {

    /**
     * Start a given JOnAS service.
     * @param serviceName the service name
     */
    void startService(String serviceName);

    /**
     * Stop a given JOnAS service.
     * @param serviceName the service name
     */
    void stopService(String serviceName);

    /**
     * Compute all JOnAS service states.
     * @throws Exception If the update fails
     */
    void update() throws Exception;

    /**
     * Return the list of JOnAS services.
     * @return The list of JOnAS services
     */
    List<String> getServices();

    /**
     * Return the description of a given JOnAS service.
     * @param serviceName the service name
     * @return The description of a given JOnAS service
     */
    String getServiceDescription(final String serviceName);

    /**
     * Return the state of a given JOnAS service.
     * @param serviceName the service name
     * @return The state of a given JOnAS service
     */
    String getServiceState(final String serviceName);
}
