package org.ow2.jonas.lib.management.tools;

/**
 * The JonasManagement singleton for management tools.
 */
public class JonasManagement {

    /**
     * The JonasManagement singleton for management tools.
     */
    private static JonasManagement unique = null;

    private static ServerManagement serverManagement = null;

    private static DomainManagement domainManagement = null;

    /**
     * JonasManagement constructor
     */
    public JonasManagement() {
        serverManagement = new ServerManagement();
        domainManagement = new DomainManagement();
    }

    /**
     * @return JonasManagement singleton.
     */
    public static JonasManagement getInstance() {
        if (unique == null) {
            unique = new JonasManagement();
        }
        return unique;
    }
    // General information about the current server to which the administration tool is connected to.
    // ----------------------------------------------------------------------------------------------
    /**
     * @return The current server name.
     */
    public static String getServerName() {
        return serverManagement.getServerName();
    }
    /**
     * @return The current domain name.
     */
    public String getDomainName() {
        return serverManagement.getDomainName();
    }
    // Server level management
    // -----------------------
    // Allow management of the current server or a managed server in the domain
    //
    /**
     * Get the current server's version.
     * @return the "serverVersion" attribute of the current server's J2EEServer MBean.
     * @throws ManagementException management exception
     */
    public String getServerVersion() throws ManagementException {
        return serverManagement.getServerVersion();
    }
    /**
     * Get the version of a server in the domain.
     * @param pServer managed server's name.
     * @return the "serverVersion" attribute of the corresponding J2EEServer MBean.
     * @throws ManagementException management exception
     */
    public static String getServerVersion(final String pServer) throws ManagementException {
        return serverManagement.getServerVersion(pServer);
    }
    /**
     * Get the memory used by the current server.
     * @return the amount of memory used by the current server
     * @throws ManagementException management exception
     */
    public long currentUsedMemory() throws ManagementException {
        return serverManagement.currentUsedMemory();
    }

    /**
     * Get the memory used by a server in the domain.
     * @return the amount of memory used by a server in the domain
     * @param pServer managed server's name.
     * @throws ManagementException management exception
     */
    public long currentUsedMemory(final String pServer) throws ManagementException {
        return serverManagement.currentUsedMemory(pServer);

    }

    // ----------------------------
    /**
     * Return True is the managed server is a master.
     * @param pServer managed server name
     * @return the "master" attribute of the corresponding J2EEDomain MBean
     */
    public boolean isMaster(final String pServer) {
        return domainManagement.isMaster(pServer);
    }
    /**
     * Return True is the current server is a master.
     * @return the "master" attribute of the current J2EEDomain MBean
     */
    public boolean isMaster() {
        return domainManagement.isMaster();
    }
    // Domain level management
    // -----------------------

    /**
     * Return the server names that are belonging to the domain.
     * @return the server names that are belonging to the domain
     */
    public String[] getServerNames() {
        return domainManagement.getServerNames();
    }
}
