/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.security.rules;

import org.ow2.jonas.deployment.common.rules.JRuleSetBase;

import org.apache.commons.digester.Digester;



/**
 * Add the rules for the memory in the jonas-realm.xml file
 *
 * @author Guillaume Sauthier
 */
public class JMemoryRuleSet extends JRuleSetBase {

    /**
     * Construct an object with a specific prefix
     * @param prefix prefix to use during the parsing
     */
    public JMemoryRuleSet(String prefix) {
        super(prefix);
   }
     /**
     * Add a set of rules to the digester object
     * @param digester Digester instance
     */
    public void addRuleInstances(Digester digester) {
        // Configure the actions we will be using
        digester.addObjectCreate(prefix + "memoryrealm",
                                 "org.ow2.jonas.security.internal.realm.factory.JResourceMemory");

        digester.addSetProperties(prefix + "memoryrealm");
        digester.addSetNext(prefix + "memoryrealm",
                            "addJResource",
                            "org.ow2.jonas.security.realm.factory.JResource");

        digester.addObjectCreate(prefix + "memoryrealm/roles/role",
                                 "org.ow2.jonas.security.internal.realm.principal.Role");
        digester.addSetProperties(prefix + "memoryrealm/roles/role");
        digester.addSetNext("memoryrealm/roles/role",
                            "addRole",
                            "org.ow2.jonas.security.internal.realm.principal.Role");

        digester.addObjectCreate(prefix + "memoryrealm/groups/group",
                                 "org.ow2.jonas.security.internal.realm.principal.Group");
        digester.addSetProperties(prefix + "memoryrealm/groups/group");
        digester.addSetNext(prefix + "memoryrealm/groups/group",
                            "addGroup",
                            "org.ow2.jonas.security.internal.realm.principal.Group");

        digester.addObjectCreate(prefix + "memoryrealm/users/user",
                                 "org.ow2.jonas.security.internal.realm.principal.User");
        digester.addSetProperties(prefix + "memoryrealm/users/user");
        digester.addSetNext(prefix + "memoryrealm/users/user",
                            "addUser",
                            "org.ow2.jonas.security.internal.realm.principal.User");
    }

}
