/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2007 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
*/

package org.ow2.jonas.ws.base.factory;

import org.ow2.jonas.lib.bootstrap.JProp;
import org.ow2.jonas.ws.WSServiceException;


/**
 * Used to retrieve from jonas.properties the factory to
 * use for Service creation (WebServices).
 *
 * @author Xavier Delplanque
 * @author Guillaume Sauthier
 */
public final class JServiceFactoryFinder {
    /**
     * JProp instance.
     */
    private static JProp props = null;

    /**
     * JOnAS ClassLoader.
     */
    private static ClassLoader jonasCL =
        Thread.currentThread().getContextClassLoader();

    /**
     * Property name.
     */
    private static final String JONAS_SERVICE_FACTORY =
        "jonas.service.ws.factory.class";

    /**
     * Private empty constructor for utility class.
     */
    private JServiceFactoryFinder() { }

    /**
     * Factory class name.
     */
    private static String factoryClassName = null;

    /**
     * Return the JServiceFactory specified in jonas.properties.
     *
     * @return the JServiceFactory specified in jonas.properties
     *
     * @throws WSServiceException When JServiceFactory instantiation fails
     */
    public static JServiceFactory getJOnASServiceFactory()
        throws WSServiceException {
        JServiceFactory factory = null;

        if (props == null) {
            try {
                props = JProp.getInstance();
                factoryClassName = props.getValue(JONAS_SERVICE_FACTORY);
            } catch (Exception e) {
                String err = "Error when trying to get jonas property '"
                             + JONAS_SERVICE_FACTORY + "'";
                throw new WSServiceException(err, e);
            }
        }

        if (factoryClassName == null) {
            String err = "jonas property '" + JONAS_SERVICE_FACTORY + "' must be set!";
            throw new WSServiceException(err);
        }

        // Instantiation
        try {
            factory = (JServiceFactory) jonasCL.loadClass(factoryClassName)
                                               .newInstance();
        } catch (ClassNotFoundException cnfe) {
            String err = "ClassNotFound '" + factoryClassName + "' in JOnAS ClassLoader";
            throw new WSServiceException(err);
        } catch (InstantiationException ie) {
            String err = "Instantiation error for new '" + factoryClassName + "'";
            throw new WSServiceException(err);
        } catch (IllegalAccessException ie) {
            String err = "Illegal Access for new '" + factoryClassName + "'";
            throw new WSServiceException(err);
        }

        return factory;

    }
}
